@@ -10,4 +10,8 @@ def test_rsa():
     decrypt_content = rsa_decrypt(client_private_pem, encrypt_content.decode('utf-8'), secret)
     assert rsa_verify(server_public_pem, encrypt_content.decode('utf-8'), sign_content.decode('utf-8'), secret) == True
 
+
+def test_dsa():
+    pass
+
 test_rsa()
\ No newline at end of file
